@@ -168,6 +168,11 @@ folder_format_str (char *dest, size_t destlen, char op, const char *src,
       break;
       
     case 'f':
+#ifdef USE_IMAP
+      if (mx_is_imap (folder->ff->name))
+        strfcpy (fn, NONULL(folder->ff->desc), sizeof (fn));
+      else
+#endif
       strfcpy (fn, folder->ff->name, sizeof(fn));
       if (folder->ff->st != NULL)
       {
@@ -199,10 +204,10 @@ folder_format_str (char *dest, size_t destlen, char op, const char *src,
       else
       {
 #ifdef USE_IMAP
-	if (strchr(folder->ff->name, '{'))
+	if (mx_is_imap (folder->ff->name))
 	{
-	  snprintf (tmp, sizeof (tmp), "%%%ss", fmt);
-	  snprintf (dest, destlen, tmp, "IMAP");
+          snprintf (tmp, sizeof (tmp), "%%%ss", fmt);
+          snprintf (dest, destlen, tmp, "IMAP");
 	}                                        
 #endif
       }
@@ -297,8 +302,6 @@ folder_format_str (char *dest, size_t destlen, char op, const char *src,
 static void add_folder (MUTTMENU *m, struct browser_state *state,
 			const char *name, const struct stat *s, int new)
 {
-  char buffer[_POSIX_PATH_MAX + SHORT_STRING];
-
   if (state->entrylen == state->entrymax)
   {
     /* need to allocate more space */
@@ -320,7 +323,7 @@ static void add_folder (MUTTMENU *m, struct browser_state *state,
 
   (state->entry)[state->entrylen].is_new = new;
   (state->entry)[state->entrylen].name = safe_strdup (name);
-  (state->entry)[state->entrylen].desc = safe_strdup (buffer);
+  (state->entry)[state->entrylen].desc = safe_strdup (name);
 #ifdef USE_IMAP
   (state->entry)[state->entrylen].notfolder = 0;
 #endif
@@ -474,11 +477,11 @@ static void init_menu (struct browser_state *state, MUTTMENU *menu, char *title,
 #ifdef USE_IMAP
   if (state->imap_browse && option (OPTIMAPLSUB))
     snprintf (title, titlelen, _("Subscribed [%s], File mask: %s"),
-	      path, Mask.pattern);
+	      path, NONULL (Mask.pattern));
   else
 #endif
     snprintf (title, titlelen, _("Directory [%s], File mask: %s"),
-	      path, Mask.pattern);
+	      path, NONULL(Mask.pattern));
   }
   menu->redraw = REDRAW_FULL;
 }
@@ -1027,7 +1030,7 @@ void _mutt_select_file (char *f, size_t flen, int buffy,
 	  b = mutt_make_file_attach (buf);
 	  if (b != NULL)
 	  {
-	    mutt_view_attachment (NULL, b, M_REGULAR);
+	    mutt_view_attachment (NULL, b, M_REGULAR, NULL, NULL, 0);
 	    mutt_free_body (&b);
 	    menu->redraw = REDRAW_FULL;
 	  }
