@@ -240,6 +240,11 @@ int mutt_buffy_check (int force)
     else
       last_imap_check = now;
   }
+
+  /* update postponed count as well, on force */
+  if (force)
+    mutt_update_num_postponed ();
+
 #endif
 
   /* fastest return if there are no mailboxes */
