@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -26,6 +26,10 @@
 #include "mx.h"
 #include "pager.h"
 
+#ifdef USE_IMAP
+#include "imap.h"
+#endif
+
 #ifdef BUFFY_SIZE
 #include "buffy.h"
 #endif
@@ -73,7 +77,6 @@ int mutt_display_message (HEADER *cur)
   mutt_parse_mime_message (Context, cur);
 
 
-
 #ifdef _PGPPATH
   /* see if PGP is needed for this message.  if so, we should exit curses */
   if (cur->pgp)
@@ -84,7 +87,6 @@ int mutt_display_message (HEADER *cur)
 	return 0;
 
       cmflags |= M_CM_VERIFY;
-      mutt_message _("Invoking PGP...");
     }
     else if (cur->pgp & PGPSIGN)
     {
@@ -92,18 +94,20 @@ int mutt_display_message (HEADER *cur)
       if (query_quadoption (OPT_VERIFYSIG, _("Verify PGP signature?")) == M_YES)
       {
 	cmflags |= M_CM_VERIFY;
-	mutt_message _("Invoking PGP...");
       }
     }
   }
+  
+  if ((cmflags & M_CM_VERIFY) || (cur->pgp & PGPENCRYPT))
+  {
+    if (cur->env->from)
+      pgp_invoke_getkeys (cur->env->from);
+
+    mutt_message _("Invoking PGP...");
+  }
+
 #endif
 
-
-
-
-
-
-
   mutt_mktemp (tempfile);
   if ((fpout = safe_fopen (tempfile, "w")) == NULL)
   {
@@ -135,10 +139,20 @@ int mutt_display_message (HEADER *cur)
     return (0);
   }
 
+#ifdef _PGPPATH
+  /* update PGP information for this message */
+  cur->pgp |= pgp_query (cur->content);
+#endif
+
   if (builtin)
   {
     pager_t info;
-    
+
+#ifdef _PGPPATH
+    if (cur->pgp & PGPGOODSIGN)
+      mutt_message _("PGP signature successfully verified.");
+#endif
+
     /* Invoke the builtin pager */
     memset (&info, 0, sizeof (pager_t));
     info.hdr = cur;
@@ -213,18 +227,33 @@ void ci_bounce_message (HEADER *h, int *redraw)
     return;
   }
 
-  mutt_bounce_message (h, adr);
+  mutt_bounce_message (NULL, h, adr);
   rfc822_free_address (&adr);
   mutt_message (h ? _("Message bounced.") : _("Messages bounced."));
 }
 
 void mutt_pipe_message_to_state (HEADER *h, STATE *s)
 {
+  int cmflags = 0;
+  int chflags = CH_FROM;
+  
+  if (option (OPTPIPEDECODE))
+  {
+    cmflags |= M_CM_DECODE | M_CM_CHARCONV;
+    chflags |= CH_DECODE | CH_REORDER;
+    
+    if (option (OPTWEED))
+    {
+      chflags |= CH_WEED;
+      cmflags |= M_CM_WEED;
+    }
+  }
+  
   if (option (OPTPIPEDECODE))
     mutt_parse_mime_message (Context, h);
+
   mutt_copy_message (s->fpout, Context, h,
-		     option (OPTPIPEDECODE) ? M_CM_DECODE | M_CM_CHARCONV: 0,
-		     option (OPTPIPEDECODE) ? CH_FROM | CH_WEED | CH_DECODE | CH_REORDER : CH_FROM);
+		     cmflags, chflags);
 }
 
 int mutt_pipe_message (HEADER *h)
@@ -437,59 +466,76 @@ void mutt_enter_command (void)
     set_option (OPTNEEDRESORT);
 }
 
-void mutt_display_address (ADDRESS *adr)
+void mutt_display_address (ENVELOPE *env)
 {
+  char *pfx = NULL;
   char buf[SHORT_STRING];
+  ADDRESS *adr = NULL;
+
+  adr = mutt_get_address (env, &pfx);
+
+  if (!adr) return;
 
   buf[0] = 0;
   rfc822_write_address (buf, sizeof (buf), adr);
-  mutt_message ("%s", buf);
+  mutt_message ("%s: %s", pfx, buf);
 }
 
-static void set_copy_flags(HEADER *hdr, int decode, int decrypt, int *cmflags, int *chflags)
+static void set_copy_flags (HEADER *hdr, int decode, int decrypt, int *cmflags, int *chflags)
 {
   *cmflags = 0;
   *chflags = CH_UPDATE_LEN;
   
 #ifdef _PGPPATH
-  if(!decode && decrypt && (hdr->pgp & PGPENCRYPT))
+  if (!decode && decrypt && (hdr->pgp & PGPENCRYPT))
   {
-    if(mutt_is_multipart_encrypted(hdr->content))
+    if (mutt_is_multipart_encrypted(hdr->content))
     {
       *chflags = CH_NONEWLINE | CH_XMIT | CH_MIME;
       *cmflags = M_CM_DECODE_PGP;
     }
-    else if(mutt_is_application_pgp(hdr->content) & PGPENCRYPT)
+    else if (mutt_is_application_pgp(hdr->content) & PGPENCRYPT)
       decode = 1;
   }
 #endif
 
-  if(decode)
+  if (decode)
   {
     *chflags = CH_XMIT | CH_MIME | CH_TXTPLAIN;
     *cmflags = M_CM_DECODE | M_CM_CHARCONV;
   }
 
+  /* respect $weed only if decode doesn't kick in
+   * for decrypt.
+   */
+
+  if (decode && !decrypt && option (OPTWEED))
+  {
+    *chflags |= CH_WEED;
+    *cmflags |= M_CM_WEED;
+  }
 }
 
 static void _mutt_save_message (HEADER *h, CONTEXT *ctx, int delete, int decode, int decrypt)
 {
   int cmflags, chflags;
   
-  set_copy_flags(h, decode, decrypt, &cmflags, &chflags);
+  set_copy_flags (h, decode, decrypt, &cmflags, &chflags);
 
-  if (decode  || decrypt)
+  if (decode || decrypt)
     mutt_parse_mime_message (Context, h);
 
   if (mutt_append_message (ctx, Context, h, cmflags, chflags) == 0 && delete)
   {
     mutt_set_flag (Context, h, M_DELETE, 1);
-    mutt_set_flag (Context, h, M_TAG, 0);
+    if (option (OPTDELETEUNTAG))
+      mutt_set_flag (Context, h, M_TAG, 0);
   }
 }
 
 /* returns 0 if the copy/save was successful, or -1 on error/abort */
-int mutt_save_message (HEADER *h, int delete, int decode, int decrypt, int *redraw)
+int mutt_save_message (HEADER *h, int delete, 
+		       int decode, int decrypt, int *redraw)
 {
   int i, need_buffy_cleanup;
 #ifdef _PGPPATH
@@ -580,6 +626,22 @@ int mutt_save_message (HEADER *h, int delete, int decode, int decrypt, int *redr
   
   mutt_message (_("Copying to %s..."), buf);
   
+#ifdef USE_IMAP
+  if (Context->magic == M_IMAP && 
+      !(decode || decrypt) && mx_is_imap (buf))
+  {
+    switch (imap_copy_messages (Context, h, buf, delete))
+    {
+      /* success */
+      case 0: mutt_clear_error (); return 0;
+      /* non-fatal error: fall through to fetch/append */
+      case 1: break;
+      /* fatal error, abort */
+      case -1: return -1;
+    }
+  }
+#endif
+
   if (mx_open_mailbox (buf, M_APPEND, &ctx) != NULL)
   {
     if (h)
@@ -625,8 +687,14 @@ int mutt_save_message (HEADER *h, int delete, int decode, int decrypt, int *redr
 
 static void print_msg (FILE *fp, CONTEXT *ctx, HEADER *h)
 {
-  
+  int cmflags = M_CM_DECODE | M_CM_CHARCONV;
+  int chflags = CH_DECODE | CH_REORDER;
 
+  if (option (OPTWEED))
+  {
+    cmflags |= M_CM_WEED;
+    chflags |= CH_WEED;
+  }
 
 #ifdef _PGPPATH
   if (h->pgp & PGPENCRYPT)
@@ -637,10 +705,8 @@ static void print_msg (FILE *fp, CONTEXT *ctx, HEADER *h)
   }
 #endif
 
-
-
   mutt_parse_mime_message (ctx, h);
-  mutt_copy_message (fp, ctx, h, M_CM_DECODE | M_CM_CHARCONV, CH_WEED | CH_DECODE | CH_REORDER);
+  mutt_copy_message (fp, ctx, h, cmflags, chflags);
 }
 
 void mutt_print_message (HEADER *h)
