@@ -133,8 +133,8 @@ void mutt_edit_file (const char *editor, const char *data)
 int mutt_yesorno (const char *msg, int def)
 {
   event_t ch;
-  char *yes = _("yes");
-  char *no = _("no");
+  unsigned char *yes = (unsigned char *) _("yes");
+  unsigned char *no = (unsigned char *) _("no");
   
   CLEARLINE(LINES-1);
   printw("%s ([%c]/%c): ", msg, def ? *yes : *no,
@@ -161,7 +161,7 @@ int mutt_yesorno (const char *msg, int def)
       BEEP();
     }
   }
-  addstr (def ? yes : no);
+  addstr ((char *) (def ? yes : no));
   mutt_refresh ();
   return (def);
 }
@@ -247,7 +247,7 @@ void mutt_endwin (const char *msg)
 {
   if (!option (OPTNOCURSES))
   {
-#ifdef SLANG_CURSES
+#ifdef USE_SLANG_CURSES
     CLEARLINE (LINES - 1);
     move (LINES - 1, 0)
 #else
@@ -258,7 +258,7 @@ void mutt_endwin (const char *msg)
     attrset (A_NORMAL);
     mutt_refresh ();
     endwin ();
-#ifndef SLANG_CURSES
+#ifndef USE_SLANG_CURSES
     fputc ('\n', stdout);
 #endif
   }
