@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -24,6 +24,11 @@
 #include "sort.h"
 #include "buffy.h"
 
+#ifdef USE_IMAP
+#include "mx.h"
+#include "imap.h"
+#endif
+
 
 
 #ifdef _PGPPATH
@@ -284,7 +289,8 @@ int mutt_index_menu (void)
   int do_buffy_notify = 1;
   int close = 0; /* did we OP_QUIT or OP_EXIT out of this menu? */
   int attach_msg = option(OPTATTACHMSG);
-
+  int check_lock = 0;
+  
   menu = mutt_new_menu ();
   menu->menu = MENU_MAIN;
   menu->offset = 1;
@@ -320,7 +326,7 @@ int mutt_index_menu (void)
        */
       index_hint = (Context->vcount) ? CURHDR->index : 0;
 
-      if ((check = mx_check_mailbox (Context, &index_hint)) < 0)
+      if ((check = mx_check_mailbox (Context, &index_hint, check_lock)) < 0)
       {
 	if (!Context->path)
 	{
@@ -449,6 +455,8 @@ int mutt_index_menu (void)
       }
     }
 
+    check_lock = 0;
+    
     if (!attach_msg)
     {
      /* check for new mail in the incoming folders */
@@ -744,6 +752,11 @@ int mutt_index_menu (void)
 
 	if (query_quadoption (OPT_QUIT, _("Quit Mutt?")) == M_YES)
 	{ 
+#ifdef USE_IMAP
+          /* logout gracefully from the IMAP server */
+          if (Context && Context->magic == M_IMAP)
+            imap_set_logout (Context);
+#endif
 	  if (!Context || mx_close_mailbox (Context) == 0)
 	    done = 1;
 	  else
@@ -784,6 +797,10 @@ int mutt_index_menu (void)
 
       case OP_MAIN_SYNC_FOLDER:
 
+        check_lock = 1;	/* call mx_check_mailbox() with locking the 
+			 * nex time
+			 */
+
 	CHECK_MSGCOUNT;
 	CHECK_READONLY;
 	{
@@ -981,6 +998,10 @@ int mutt_index_menu (void)
 	{
 	  if (Context)
 	  {
+#ifdef USE_IMAP
+            if (Context->magic == M_IMAP)
+              imap_set_logout (Context);
+#endif
 	    mx_fastclose_mailbox (Context);
 	    safe_free ((void **) &Context);
 	  }
@@ -1145,7 +1166,7 @@ int mutt_index_menu (void)
 	    }
 	  }
 
-	  if ((Sort & SORT_MASK) == SORT_THREADS && CUR->collapsed)
+	  if (CUR->collapsed && (Sort & SORT_MASK) == SORT_THREADS)
 	  {
 	    if ((op == OP_MAIN_NEXT_UNREAD || op == OP_MAIN_PREV_UNREAD) &&
 		UNREAD (CUR))
@@ -1189,20 +1210,34 @@ int mutt_index_menu (void)
 
 	CHECK_MSGCOUNT;
 	CHECK_READONLY;
-	mutt_set_flag (Context, CURHDR, M_FLAG, !CURHDR->flagged);
 
-	if (option (OPTRESOLVE))
-	{
-	  if ((menu->current = ci_next_undeleted (menu->current)) == -1)
+        if (tag)
+        {
+	  for (j = 0; j < Context->vcount; j++)
 	  {
-	    menu->current = menu->oldcurrent;
-	    menu->redraw = REDRAW_CURRENT;
+	    if (Context->hdrs[Context->v2r[j]]->tagged)
+	      mutt_set_flag (Context, Context->hdrs[Context->v2r[j]],
+			     M_FLAG, !Context->hdrs[Context->v2r[j]]->flagged);
+	  }
+
+	  menu->redraw |= REDRAW_INDEX;
+	}
+        else
+        {
+	  mutt_set_flag (Context, CURHDR, M_FLAG, !CURHDR->flagged);
+	  if (option (OPTRESOLVE))
+	  {
+	    if ((menu->current = ci_next_undeleted (menu->current)) == -1)
+	    {
+	      menu->current = menu->oldcurrent;
+	      menu->redraw = REDRAW_CURRENT;
+	    }
+	    else
+	      menu->redraw = REDRAW_MOTION_RESYNCH;
 	  }
 	  else
-	    menu->redraw = REDRAW_MOTION_RESYNCH;
+	    menu->redraw = REDRAW_CURRENT;
 	}
-	else
-	  menu->redraw = REDRAW_CURRENT;
 	menu->redraw |= REDRAW_STATUS;
 	break;
 
@@ -1297,6 +1332,23 @@ int mutt_index_menu (void)
 	  menu->redraw = REDRAW_MOTION;
 	break;
 
+      case OP_MAIN_PARENT_MESSAGE:
+
+	CHECK_MSGCOUNT;
+
+	if ((menu->current = mutt_parent_message (Context, CURHDR)) < 0)
+	{
+	  menu->current = menu->oldcurrent;
+	}
+	else if (menu->menu == MENU_PAGER)
+        {
+          op = OP_DISPLAY_MESSAGE;
+          continue;
+        }
+        else
+          menu->redraw = REDRAW_MOTION;
+	break;
+
       case OP_MAIN_SET_FLAG:
       case OP_MAIN_CLEAR_FLAG:
 
@@ -1427,11 +1479,15 @@ int mutt_index_menu (void)
 	if (tag)
 	{
 	  mutt_tag_set_flag (M_DELETE, 1);
+	  if (option (OPTDELETEUNTAG))
+	    mutt_tag_set_flag (M_TAG, 0);
 	  menu->redraw = REDRAW_INDEX;
 	}
 	else
 	{
 	  mutt_set_flag (Context, CURHDR, M_DELETE, 1);
+	  if (option (OPTDELETEUNTAG))
+	    mutt_set_flag (Context, CURHDR, M_TAG, 0);
 	  if (option (OPTRESOLVE))
 	  {
 	    if ((menu->current = ci_next_undeleted (menu->current)) == -1)
@@ -1474,7 +1530,7 @@ int mutt_index_menu (void)
       case OP_DISPLAY_ADDRESS:
 
 	CHECK_MSGCOUNT;
-	mutt_display_address (CURHDR->env->from);
+	mutt_display_address (CURHDR->env);
 	break;
 
       case OP_ENTER_COMMAND:
@@ -1493,11 +1549,10 @@ int mutt_index_menu (void)
 	CHECK_MSGCOUNT;
 	CHECK_READONLY;
 	CHECK_ATTACH;
-        
-        set_option(OPTUSEHEADERDATE);
-	ci_send_message (SENDEDITMSG, NULL, NULL, Context, CURHDR);
-        unset_option(OPTUSEHEADERDATE);
+
+        mutt_edit_message (Context, tag ? NULL : CURHDR);
 	menu->redraw = REDRAW_FULL;
+
 	break;
 
       case OP_FORWARD_MESSAGE:
@@ -1611,6 +1666,25 @@ int mutt_index_menu (void)
 	menu->redraw = REDRAW_FULL;
 	break;
 
+      case OP_RESEND:
+      
+        CHECK_ATTACH;
+        CHECK_MSGCOUNT;
+      
+        if (tag)
+        {
+	  for (j = 0; j < Context->vcount; j++)
+	  {
+	    if (Context->hdrs[Context->v2r[j]]->tagged)
+	      mutt_resend_message (NULL, Context, Context->hdrs[Context->v2r[j]]);
+	  }
+	}
+        else
+	  mutt_resend_message (NULL, Context, CURHDR);
+      
+        menu->redraw = REDRAW_FULL;
+        break;
+      
       case OP_REPLY:
 
 	CHECK_ATTACH;
@@ -1714,7 +1788,9 @@ int mutt_index_menu (void)
     {
       menu->menu = MENU_MAIN;
       menu->redraw = REDRAW_FULL;
+#if 0
       set_option (OPTWEED); /* turn header weeding back on. */
+#endif
     }
 
     if (done) break;
