@@ -21,19 +21,24 @@
 /* general IMAP utility functions */
 
 #include "mutt.h"
-#include "imap.h"
 #include "imap_private.h"
 
 #include <stdlib.h>
 #include <ctype.h>
 
-/* imap_error: handle IMAP errors. Should be expanded to display the error
- *   to the user and ask whether to continue, return result code to caller */
-int imap_error (const char *where, const char *msg)
+/* imap_continue: display a message and ask the user if she wants to
+ *   go on. */
+int imap_continue (const char* msg, const char* resp)
+{
+  imap_error (msg, resp);
+  return mutt_yesorno (_("Continue?"), 0);
+}
+
+/* imap_error: show an error and abort */
+void imap_error (const char *where, const char *msg)
 {
   mutt_error (_("%s [%s]\n"), where, msg);
   sleep (2);
-  return mutt_yesorno (_("Continue?"), 0);
 }
 
 /*
@@ -90,15 +95,18 @@ int imap_get_literal_count(const char *buf, long *bytes)
   return (0);
 }
 
-/* imap_make_sequence: make a tag suitable for starting an IMAP command */
-void imap_make_sequence (char *buf, size_t buflen)
+/* imap_get_qualifier: in a tagged response, skip tag and status for
+ *   the qualifier message. Used by imap_copy_message for TRYCREATE */
+char* imap_get_qualifier (char* buf)
 {
-  static int sequence = 0;
-  
-  snprintf (buf, buflen, "a%04d", sequence++);
+  char *s = buf;
 
-  if (sequence > 9999)
-    sequence = 0;
+  /* skip tag */
+  s = imap_next_word (s);
+  /* skip OK/NO/BAD response */
+  s = imap_next_word (s);
+
+  return s;
 }
 
 /* imap_next_word: return index into string where next IMAP word begins */
@@ -123,23 +131,34 @@ int imap_parse_path (char* path, char* host, size_t hlen, int* port,
   *port = IMAP_PORT;
   pc = path;
   if (*pc != '{')
-    return (-1);
+    return -1;
   pc++;
   n = 0;
   while (*pc && *pc != '}' && *pc != ':' && (n < hlen-1))
     host[n++] = *pc++;
   host[n] = 0;
+  /* catch NULL hosts */
+  if (!*host)
+  {
+    dprint (1, (debugfile, "imap_parse_path: NULL host in %s\n", path));
+    return -1;
+  }
   if (!*pc)
-    return (-1);
+    return -1;
   if (*pc == ':')
   {
     pc++;
     pt = pc;
     while (*pc && *pc != '}') pc++;
     if (!*pc)
-      return (-1);
+      return -1;
     *pc = '\0';
     *port = atoi (pt);
+    if (!port)
+    {
+      dprint (1, (debugfile, "imap_parse_path: bad port in %s\n", path));
+      return -1;
+    }
     *pc = '}';
   }
   pc++;
@@ -161,8 +180,8 @@ void imap_qualify_path (char* dest, size_t len, const char* host, int port,
       NONULL (name));
 }
 
-/* imap_quote_string: quote string according to vague IMAP rules. Not
- *   currently very smart. */
+/* imap_quote_string: quote string according to IMAP rules:
+ *   surround string with quotes, escape " and \ with \ */
 void imap_quote_string (char *dest, size_t slen, const char *src)
 {
   char quote[] = "\"\\", *pt;
@@ -226,3 +245,25 @@ void imap_unquote_string (char *s)
   }
   *d = '\0';
 }
+
+/* imap_wordcasecmp: find word a in word list b */
+int imap_wordcasecmp(const char *a, const char *b)
+{
+  char tmp[SHORT_STRING];
+  char *s = (char *)b;
+  int i;
+
+  tmp[SHORT_STRING-1] = 0;
+  for(i=0;i < SHORT_STRING-2;i++,s++)
+  {
+    if (!*s || ISSPACE(*s))
+    {
+      tmp[i] = 0;
+      break;
+    }
+    tmp[i] = *s;
+  }
+  tmp[i+1] = 0;
+
+  return mutt_strcasecmp(a, tmp);
+}
