@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -38,12 +38,14 @@ struct mapping_t Menus[] = {
  { "pager",	MENU_PAGER },
  { "postpone",	MENU_POST },
 
-  
 
 #ifdef _PGPPATH
  { "pgp",	MENU_PGP },
 #endif  
   
+#ifdef MIXMASTER
+  { "mix", 	MENU_MIX },
+#endif
   
 
  { "query",	MENU_QUERY },
@@ -378,8 +380,8 @@ int km_dokey (int menu)
       struct binding_t *bindings;
 
       /* is this a valid op for this menu? */
-      if ((bindings = km_get_table (menu)) &&
-	  (func = get_func (bindings, tmp.op)))
+      bindings = km_get_table (menu);
+      if ((func = get_func (bindings, tmp.op)))
 	return tmp.op;
 
       if (menu == MENU_EDITOR && get_func (OpEditor, tmp.op))
@@ -540,7 +542,13 @@ void km_init (void)
   create_bindings (OpPgp, MENU_PGP);
 #endif
 
-
+#ifdef MIXMASTER
+  create_bindings (OpMix, MENU_MIX);
+  
+  km_bindkey ("<space>", MENU_MIX, OP_GENERIC_SELECT_ENTRY);
+  km_bindkey ("h", MENU_MIX, OP_MIX_CHAIN_PREV);
+  km_bindkey ("l", MENU_MIX, OP_MIX_CHAIN_NEXT);
+#endif
   
   /* bindings for the line editor */
   create_bindings (OpEditor, MENU_EDITOR);
@@ -612,8 +620,9 @@ void km_init (void)
 
   km_bindkey ("<enter>", MENU_PAGER, OP_NEXT_LINE);
   
-  km_bindkey ("<return>", MENU_ALIAS, OP_TAG);
-  km_bindkey ("<enter>",  MENU_ALIAS, OP_TAG);
+  km_bindkey ("<return>", MENU_ALIAS, OP_GENERIC_SELECT_ENTRY);
+  km_bindkey ("<enter>",  MENU_ALIAS, OP_GENERIC_SELECT_ENTRY);
+  km_bindkey ("<space>", MENU_ALIAS, OP_TAG);
 
   km_bindkey ("<enter>", MENU_ATTACH, OP_VIEW_ATTACH);
   km_bindkey ("<enter>", MENU_COMPOSE, OP_VIEW_ATTACH);
@@ -745,6 +754,10 @@ struct binding_t *km_get_table (int menu)
 #endif
 
 
+#ifdef MIXMASTER
+    case MENU_MIX:
+      return OpMix;
+#endif
 
   }
   return NULL;
