@@ -1,5 +1,6 @@
 /*
- * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1999   Thomas Roessler <roessler@guug.de>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -36,20 +37,21 @@
 #include <getopt.h>
 #endif
 
-const char *ReachingUs = N_("\
+static const char *ReachingUs = N_("\
 To contact the developers, please mail to <mutt-dev@mutt.org>.\n");
 
-const char *Notice = N_("\
-Copyright (C) 1996-2000 Michael R. Elkins and others.\n\
+static const char *Notice = N_("\
+Copyright (C) 1996-9 Michael R. Elkins and others.\n\
 Mutt comes with ABSOLUTELY NO WARRANTY; for details type `mutt -vv'.\n\
 Mutt is free software, and you are welcome to redistribute it\n\
 under certain conditions; type `mutt -vv' for details.\n");
 
-const char *Copyright = N_("\
-Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>\n\
-Copyright (C) 1997-2000 Thomas Roessler <roessler@guug.de>\n\
-Copyright (C) 1998-2000 Werner Koch <wk@isil.d.shuttle.de>\n\
-Copyright (C) 1998-2000 Ruslan Ermilov <ru@ucb.crimea.ua>\n\
+static const char *Copyright = N_("\
+Copyright (C) 1996-9 Michael R. Elkins <me@cs.hmc.edu>\n\
+Copyright (C) 1996-9 Brandon Long <blong@fiction.net>\n\
+Copyright (C) 1997-9 Thomas Roessler <roessler@guug.de>\n\
+Copyright (C) 1998-9 Werner Koch <wk@isil.d.shuttle.de>\n\
+Copyright (C) 1999   Brendan Cully <brendan@kublai.com>\n\
 \n\
 Lots of others not mentioned here contributed lots of code,\n\
 fixes, and suggestions.\n\
@@ -70,8 +72,8 @@ fixes, and suggestions.\n\
 ");
 #ifdef _PGPPATH
 
-const char *ShaCopyright = N_("\n\
-SHA1 implementation Copyright (C) 1995-1997 Eric A. Young <eay@cryptsoft.com>\n\
+const char* ShaCopyright = N_("\n\
+SHA1 implementation Copyright (C) 1995-7 Eric A. Young <eay@cryptsoft.com>\n\
 \n\
     Redistribution and use in source and binary forms, with or without\n\
     modification, are permitted under certain conditions.\n\
@@ -130,7 +132,7 @@ static void show_version (void)
   struct utsname uts;
 
   printf ("Mutt %s (%s)\n", MUTT_VERSION, ReleaseDate);
-  puts (_(Notice));
+  puts (Notice);
 
   uname (&uts);
 
@@ -195,6 +197,12 @@ static void show_version (void)
         "-USE_IMAP  "
 #endif
 
+#ifdef USE_GSS
+	"+USE_GSS  "
+#else
+	"-USE_GSS  "
+#endif
+
 #ifdef USE_POP
 	"+USE_POP  "
 #else
@@ -223,18 +231,17 @@ static void show_version (void)
   puts (
 
 #ifdef _PGPPATH
+	"\n"
 #ifdef HAVE_PGP5
 	"+HAVE_PGP5  "
 #endif
 #ifdef HAVE_PGP2
 	"+HAVE_PGP2  "
 #endif
-#ifdef HAVE_PGP6
-       "+HAVE_PGP6  "
-#endif
 #ifdef HAVE_GPG
 	"+HAVE_GPG  "
 #endif
+	"\n"
 #endif
 
 
@@ -365,13 +372,15 @@ int main (int argc, char **argv)
   textdomain (PACKAGE);
 #endif
 
+  setlocale (LC_CTYPE, "");
+
   mutt_error = mutt_nocurses_error;
   SRAND (time (NULL));
-  setlocale (LC_CTYPE, "");
   umask (077);
 
   memset (Options, 0, sizeof (Options));
-
+  memset (QuadOptions, 0, sizeof (QuadOptions));
+  
   while ((i = getopt (argc, argv, "a:b:F:f:c:d:e:H:s:i:hm:npRvxyzZ")) != EOF)
     switch (i)
     {
@@ -510,6 +519,30 @@ int main (int argc, char **argv)
     mutt_error = mutt_curses_error;
   }
 
+  /* Create the Maildir directory if it doesn't exist. */
+  if (!option (OPTNOCURSES) && Maildir)
+  {
+    struct stat sb;
+    char fpath[_POSIX_PATH_MAX];
+    char msg[STRING];
+
+    strfcpy (fpath, Maildir, sizeof (fpath));
+    mutt_expand_path (fpath, sizeof (fpath));
+#ifdef USE_IMAP
+    /* we're not connected yet - skip mail folder creation */
+    if (!mx_is_imap (fpath))
+#endif
+    if (stat (fpath, &sb) == -1 && errno == ENOENT)
+    {
+      snprintf (msg, sizeof (msg), _("%s does not exist. Create it?"), Maildir);
+      if (mutt_yesorno (msg, 1) == 1)
+      {
+	if (mkdir (fpath, 0700) == -1 && errno != EEXIST)
+	  mutt_error ( _("Can't create %s: %s."), Maildir, strerror (errno));
+      }
+    }
+  }
+
   if (sendflags & SENDPOSTPONED)
   {
     if (!option (OPTNOCURSES))
@@ -540,7 +573,7 @@ int main (int argc, char **argv)
       for (i = optind; i < argc; i++)
 	msg->env->to = rfc822_parse_adrlist (msg->env->to, argv[i]);
 
-      if (!msg->env->to && !msg->env->cc)
+      if (option (OPTAUTOEDIT) && !msg->env->to && !msg->env->cc)
       {
 	if (!option (OPTNOCURSES))
 	  mutt_endwin (NULL);
@@ -576,7 +609,7 @@ int main (int argc, char **argv)
       tempfile = safe_strdup (buf);
 
       if (draftFile)
-	msg->env = mutt_read_rfc822_header (fin, NULL, 1);
+	msg->env = mutt_read_rfc822_header (fin, NULL, 1, 0);
 
       if (tempfile)
       {
@@ -684,10 +717,8 @@ int main (int argc, char **argv)
       mutt_index_menu ();
       if (Context)
 	safe_free ((void **)&Context);
-      mutt_endwin (NULL);
     }
-    else
-      mutt_endwin (Errorbuf);
+    mutt_endwin (Errorbuf);
   }
 
   exit (0);
