@@ -245,6 +245,7 @@ enum
   OPT_ABORT,
   OPT_RECALL,
   OPT_SUBJECT,
+  OPT_MIMEFWDREST,
   OPT_MAX
 };
 
@@ -284,7 +285,6 @@ enum
   OPTFCCATTACH,
   OPTFOLLOWUPTO,
   OPTFORCENAME,
-  OPTFORWATTACH,
   OPTFORWDECODE,
   OPTFORWQUOTE,
   OPTHDRS,
