@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -38,6 +38,8 @@ typedef struct
   HEADER *hdr;	/* current message */
   BODY *bdy;	/* current attachment */
   FILE *fp;	/* source stream */
+  ATTACHPTR **idx;	/* attachment information */
+  short idxlen;
 } pager_t;
 
 int mutt_do_pager (const char *, const char *, int, pager_t *);
