@@ -1,6 +1,6 @@
 /*
- * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>
- * Copyright (C) 1999-2000Thomas Roessler <roessler@guug.de>
+ * Copyright (C) 1996-1998 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1999 Thomas Roessler <roessler@guug.de>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -121,6 +121,7 @@ ATTACHPTR **mutt_gen_attach_list (BODY *m,
       safe_realloc ((void **) &idx, sizeof (ATTACHPTR *) * (*idxmax += 5));
 
     if (m->type == TYPEMULTIPART && m->parts
+	&& (compose || (parent_type == -1 && mutt_strcasecmp ("alternative", m->subtype)))
 #ifdef _PGPPATH
 	&& !mutt_is_multipart_encrypted(m)
 #endif
@@ -138,7 +139,13 @@ ATTACHPTR **mutt_gen_attach_list (BODY *m,
       m->tagged = 0;
 
       /* We don't support multipart messages in the compose menu yet */
-      if (!compose && mutt_is_message_type(m->type, m->subtype))
+      if (!compose && 
+	  ((m->type == TYPEMULTIPART
+#ifdef _PGPPATH
+	    && !mutt_is_multipart_encrypted (m)
+#endif
+	    )
+	   || mutt_is_message_type(m->type, m->subtype)))
       {
 	idx = mutt_gen_attach_list (m->parts, m->type, idx, idxlen, idxmax, level + 1, compose);
       }
@@ -151,7 +158,9 @@ ATTACHPTR **mutt_gen_attach_list (BODY *m,
   return (idx);
 }
 
-/* %D = deleted flag
+/* %c = character set: convert?
+ * %C = character set
+ * %D = deleted flag
  * %d = description
  * %e = MIME content-transfer-encoding
  * %f = filename
@@ -174,12 +183,38 @@ const char *mutt_attach_fmt (char *dest,
 {
   char fmt[16];
   char tmp[SHORT_STRING];
+  char charset[SHORT_STRING];
   ATTACHPTR *aptr = (ATTACHPTR *) data;
   int optional = (flags & M_FORMAT_OPTIONAL);
   size_t l;
   
   switch (op)
   {
+    case 'C':
+      if (!optional)
+      {
+	snprintf (fmt, sizeof (fmt), "%%%ss", prefix);
+	if (mutt_is_text_type (aptr->content->type, aptr->content->subtype) &&
+	    mutt_get_send_charset (charset, sizeof (charset), aptr->content, 0))
+	  snprintf (dest, destlen, fmt, charset);
+	else
+	  snprintf (dest, destlen, fmt, "");
+      }
+      else if (!mutt_is_text_type (aptr->content->type, aptr->content->subtype) ||
+	       !mutt_get_send_charset (charset, sizeof (charset), aptr->content, 0))
+        optional = 0;
+      break;
+    case 'c':
+      /* XXX */
+      if (!optional)
+      {
+	snprintf (fmt, sizeof (fmt), "%%%sc", prefix);
+	snprintf (dest, destlen, fmt, aptr->content->type != TYPETEXT ||
+		  aptr->content->noconv ? 'n' : 'c');
+      }
+      else if (aptr->content->type != TYPETEXT || aptr->content->noconv)
+        optional = 0;
+      break;
     case 'd':
       if(!optional)
       {
@@ -325,7 +360,7 @@ void attach_entry (char *b, size_t blen, MUTTMENU *menu, int num)
 
 int mutt_tag_attach (MUTTMENU *menu, int n)
 {
-  return (((ATTACHPTR **) menu->data)[n]->content->tagged = !((ATTACHPTR **) menu->data)[n]->content->tagged);
+  return ((((ATTACHPTR **) menu->data)[n]->content->tagged = !((ATTACHPTR **) menu->data)[n]->content->tagged) ? 1 : -1);
 }
 
 int mutt_is_message_type (int type, const char *subtype)
@@ -484,17 +519,16 @@ mutt_query_pipe_attachment (char *command, FILE *fp, BODY *body, int filter)
   }
 }
 
-static STATE state;
-static void pipe_attachment (FILE *fp, BODY *b)
+static void pipe_attachment (FILE *fp, BODY *b, STATE *state)
 {
   FILE *ifp;
 
   if (fp)
   {
-    state.fpin = fp;
-    mutt_decode_attachment (b, &state);
+    state->fpin = fp;
+    mutt_decode_attachment (b, state);
     if (AttachSep)
-      state_puts (AttachSep, &state);
+      state_puts (AttachSep, state);
   }
   else
   {
@@ -503,27 +537,28 @@ static void pipe_attachment (FILE *fp, BODY *b)
       mutt_perror ("fopen");
       return;
     }
-    mutt_copy_stream (ifp, state.fpout);
+    mutt_copy_stream (ifp, state->fpout);
     fclose (ifp);
     if (AttachSep)
-      state_puts (AttachSep, &state);
+      state_puts (AttachSep, state);
   }
 }
 
 static void
-pipe_attachment_list (char *command, FILE *fp, int tag, BODY *top, int filter)
+pipe_attachment_list (char *command, FILE *fp, int tag, BODY *top, int filter,
+		      STATE *state)
 {
   for (; top; top = top->next)
   {
     if (!tag || top->tagged)
     {
       if (!filter && !option (OPTATTACHSPLIT))
-	pipe_attachment (fp, top);
+	pipe_attachment (fp, top, state);
       else
 	mutt_query_pipe_attachment (command, fp, top, filter);
     }
     else if (top->parts)
-      pipe_attachment_list (command, fp, tag, top->parts, filter);
+      pipe_attachment_list (command, fp, tag, top->parts, filter, state);
     if (!tag)
       break;
   }
@@ -531,6 +566,7 @@ pipe_attachment_list (char *command, FILE *fp, int tag, BODY *top, int filter)
 
 void mutt_pipe_attachment_list (FILE *fp, int tag, BODY *top, int filter)
 {
+  STATE state;
   char buf[SHORT_STRING];
   pid_t thepid;
 
@@ -550,13 +586,13 @@ void mutt_pipe_attachment_list (FILE *fp, int tag, BODY *top, int filter)
   {
     endwin ();
     thepid = mutt_create_filter (buf, &state.fpout, NULL, NULL);
-    pipe_attachment_list (buf, fp, tag, top, filter);
+    pipe_attachment_list (buf, fp, tag, top, filter, &state);
     fclose (state.fpout);
     if (mutt_wait_filter (thepid) != 0 || option (OPTWAITKEY))
       mutt_any_key_to_continue (NULL);
   }
   else
-    pipe_attachment_list (buf, fp, tag, top, filter);
+    pipe_attachment_list (buf, fp, tag, top, filter, &state);
 }
 
 static int can_print (BODY *top, int tag)
@@ -589,7 +625,7 @@ static int can_print (BODY *top, int tag)
   return (1);
 }
 
-static void print_attachment_list (FILE *fp, int tag, BODY *top)
+static void print_attachment_list (FILE *fp, int tag, BODY *top, STATE *state)
 {
   char type [STRING];
 
@@ -603,7 +639,7 @@ static void print_attachment_list (FILE *fp, int tag, BODY *top)
       {
 	if (!mutt_strcasecmp ("text/plain", top->subtype) ||
 	    !mutt_strcasecmp ("application/postscript", top->subtype))
-	  pipe_attachment (fp, top);
+	  pipe_attachment (fp, top, state);
 	else if (mutt_can_decode (top))
 	{
 	  /* decode and print */
@@ -616,10 +652,10 @@ static void print_attachment_list (FILE *fp, int tag, BODY *top)
 	  {
 	    if ((ifp = fopen (newfile, "r")) != NULL)
 	    {
-	      mutt_copy_stream (ifp, state.fpout);
+	      mutt_copy_stream (ifp, state->fpout);
 	      fclose (ifp);
 	      if (AttachSep)
-		state_puts (AttachSep, &state);
+		state_puts (AttachSep, state);
 	    }
 	  }
 	  mutt_unlink (newfile);
@@ -629,7 +665,7 @@ static void print_attachment_list (FILE *fp, int tag, BODY *top)
 	mutt_print_attachment (fp, top);
     }
     else if (top->parts)
-      print_attachment_list (fp, tag, top->parts);
+      print_attachment_list (fp, tag, top->parts, state);
     if (!tag)
       return;
   }
@@ -637,6 +673,8 @@ static void print_attachment_list (FILE *fp, int tag, BODY *top)
 
 void mutt_print_attachment_list (FILE *fp, int tag, BODY *top)
 {
+  STATE state;
+  
   pid_t thepid;
   if (query_quadoption (OPT_PRINT, tag ? _("Print tagged attachment(s)?") : _("Print attachment?")) != M_YES)
     return;
@@ -648,186 +686,24 @@ void mutt_print_attachment_list (FILE *fp, int tag, BODY *top)
     endwin ();
     memset (&state, 0, sizeof (STATE));
     thepid = mutt_create_filter (NONULL (PrintCmd), &state.fpout, NULL, NULL);
-    print_attachment_list (fp, tag, top);
+    print_attachment_list (fp, tag, top, &state);
     fclose (state.fpout);
     if (mutt_wait_filter (thepid) != 0 || option (OPTWAITKEY))
       mutt_any_key_to_continue (NULL);
   }
   else
-    print_attachment_list (fp, tag, top);
-}
-
-static void
-bounce_attachment_list (ADDRESS *adr, int tag, BODY *body, HEADER *hdr)
-{
-  for (; body; body = body->next)
-  {
-    if (!tag || body->tagged)
-    {
-      if (!mutt_is_message_type (body->type, body->subtype))
-      {
-	mutt_error _("You may only bounce message/rfc822 parts.");
-	continue;
-      }
-      body->hdr->msgno = hdr->msgno;
-      mutt_bounce_message (body->hdr, adr);
-    }
-    else if (body->parts)
-      bounce_attachment_list (adr, tag, body->parts, hdr);
-    if (!tag)
-      break;
-  }
-}
-
-static void query_bounce_attachment (int tag, BODY *top, HEADER *hdr)
-{
-  char prompt[SHORT_STRING];
-  char buf[HUGE_STRING];
-  ADDRESS *adr = NULL;
-  int rc;
-
-  buf[0] = 0;
-
-  if(!tag)
-    strfcpy(prompt, _("Bounce message to: "), sizeof(prompt));
-  else
-    strfcpy(prompt, _("Bounce tagged messages to: "), sizeof(prompt));
-
-  rc = mutt_get_field (prompt, buf, sizeof (buf), M_ALIAS);
-
-  if (rc || !buf[0])
-    return;
-
-  adr = rfc822_parse_adrlist (adr, buf);
-  adr = mutt_expand_aliases (adr);
-  buf[0] = 0;
-  rfc822_write_address (buf, sizeof (buf), adr);
-  snprintf (prompt, sizeof (prompt), tag ? _("Bounce messages to %s...?")
-	    : _("Bounce message to %s...?"), buf);
-  if (mutt_yesorno (prompt, 1) != 1)
-  {
-    rfc822_free_address (&adr);
-    CLEARLINE (LINES-1);
-    return;
-  }
-  bounce_attachment_list (adr, tag, top, hdr);
-  rfc822_free_address (&adr);
-}
-
-static void
-copy_tagged_attachments (FILE *fpout, FILE *fpin, const char *boundary, BODY *bdy)
-{
-  for (; bdy; bdy = bdy->next)
-  {
-    if (bdy->tagged)
-    {
-      fprintf (fpout, "\n--%s\n", boundary);
-      fseek (fpin, bdy->hdr_offset, 0);
-      mutt_copy_bytes (fpin, fpout, bdy->length + bdy->offset - bdy->hdr_offset);
-    }
-    else if (bdy->parts)
-      copy_tagged_attachments (fpout, fpin, boundary, bdy->parts);
-  }
-}
-
-static int
-create_tagged_message (const char *tempfile,
-		       int tag,
-		       CONTEXT *ctx,
-		       HEADER *cur,
-		       BODY *body)
-{
-  char *boundary;
-  MESSAGE *msg, *src;
-  CONTEXT tmpctx;
-  int magic;
-
-  magic = DefaultMagic;
-  DefaultMagic = M_MBOX;
-  mx_open_mailbox (tempfile, M_APPEND, &tmpctx);
-  msg = mx_open_new_message (&tmpctx, cur, M_ADD_FROM);
-  src = mx_open_message (ctx, cur->msgno);
-
-  if (tag)
-  {
-    mutt_copy_header (src->fp, cur, msg->fp, CH_XMIT, NULL);
-    boundary = mutt_get_parameter ("boundary", cur->content->parameter);
-    copy_tagged_attachments (msg->fp, src->fp, boundary, cur->content->parts);
-    fprintf (msg->fp, "--%s--\n", boundary);
-  }
-  else
-  {
-    /* single attachment */
-    mutt_copy_header (src->fp, cur, msg->fp, CH_XMIT | CH_MIME | CH_NONEWLINE, NULL);
-    fputs ("Mime-Version: 1.0\n", msg->fp);
-    mutt_write_mime_header (body, msg->fp);
-    fputc ('\n', msg->fp);
-    fseek (src->fp, body->offset, 0);
-    mutt_copy_bytes (src->fp, msg->fp, body->length);
-  }
-
-  mx_commit_message (msg, &tmpctx);
-  mx_close_message (&msg);
-  mx_close_message (&src);
-  mx_close_mailbox (&tmpctx);
-  DefaultMagic = magic;
-  return 0;
-}
-
-/* op		flag to ci_send_message()
- * tag		operate on tagged attachments?
- * hdr		current message
- * body		current attachment 
- */
-static void reply_attachment_list (int op, int tag, HEADER *hdr, BODY *body)
-{
-  HEADER *hn;
-  char tempfile[_POSIX_PATH_MAX];
-  CONTEXT *ctx;
-
-  if (!tag && body->hdr)
-  {
-    hn = body->hdr;
-    hn->msgno   = hdr->msgno; /* required for MH/maildir */
-    hn->replied = hdr->replied;
-    hn->read    = hdr->read;
-    hn->old	= hdr->old;
-    hn->changed = hdr->changed;
-    ctx = Context;
-  }
-  else
-  {
-    /* build a fake message which consists of only the tagged attachments */
-    mutt_mktemp (tempfile);
-    create_tagged_message (tempfile, tag, Context, hdr, body);
-    ctx = mx_open_mailbox (tempfile, M_QUIET, NULL);
-    hn = ctx->hdrs[0];
-  }
-
-  ci_send_message (op, NULL, NULL, ctx, hn);
-
-  if (hn->replied)
-  {
-    if (Context != ctx)
-      mutt_set_flag (Context, hdr, M_REPLIED, 1);
-    else
-      _mutt_set_flag (Context, hdr, M_REPLIED, 1, 0);
-  }
-
-  if (ctx != Context)
-  {
-    mx_fastclose_mailbox (ctx);
-    safe_free ((void **) &ctx);
-    unlink (tempfile);
-  }
+    print_attachment_list (fp, tag, top, &state);
 }
 
 void
-mutt_attach_display_loop (MUTTMENU *menu, int op, FILE *fp, ATTACHPTR **idx)
+mutt_attach_display_loop (MUTTMENU *menu, int op, FILE *fp, HEADER *hdr, 
+			  ATTACHPTR **idx, short idxlen)
 {
+#if 0
   int old_optweed = option (OPTWEED);
-
   set_option (OPTWEED);
+#endif
+  
   do
   {
     switch (op)
@@ -837,7 +713,8 @@ mutt_attach_display_loop (MUTTMENU *menu, int op, FILE *fp, ATTACHPTR **idx)
 	/* fall through */
 
       case OP_VIEW_ATTACH:
-	op = mutt_view_attachment (fp, idx[menu->current]->content, M_REGULAR);
+	op = mutt_view_attachment (fp, idx[menu->current]->content, M_REGULAR,
+				   hdr, idx, idxlen);
 	break;
 
       case OP_NEXT_ENTRY:
@@ -866,8 +743,10 @@ mutt_attach_display_loop (MUTTMENU *menu, int op, FILE *fp, ATTACHPTR **idx)
   }
   while (op != OP_NULL);
 
+#if 0
   if (option (OPTWEED) != old_optweed)
     toggle_option (OPTWEED);
+#endif
 }
 
 
@@ -950,17 +829,19 @@ void mutt_view_attachments (HEADER *hdr)
     {
       case OP_DISPLAY_HEADERS:
       case OP_VIEW_ATTACH:
-	mutt_attach_display_loop (menu, op, fp, idx);
+	mutt_attach_display_loop (menu, op, fp, hdr, idx, idxlen);
 	menu->redraw = REDRAW_FULL;
 	break;
 
       case OP_ATTACH_VIEW_MAILCAP:
-	mutt_view_attachment (fp, idx[menu->current]->content, M_MAILCAP);
+	mutt_view_attachment (fp, idx[menu->current]->content, M_MAILCAP,
+			      hdr, idx, idxlen);
 	menu->redraw = REDRAW_FULL;
 	break;
 
       case OP_ATTACH_VIEW_TEXT:
-	mutt_view_attachment (fp, idx[menu->current]->content, M_AS_TEXT);
+	mutt_view_attachment (fp, idx[menu->current]->content, M_AS_TEXT,
+			      hdr, idx, idxlen);
 	menu->redraw = REDRAW_FULL;
 	break;
 
@@ -968,7 +849,8 @@ void mutt_view_attachments (HEADER *hdr)
 
 #ifdef _PGPPATH
       case OP_EXTRACT_KEYS:
-        pgp_extract_keys_from_attachment_list (fp, menu->tagprefix, menu->tagprefix ? cur : idx[menu->current]->content);
+        pgp_extract_keys_from_attachment_list (fp, menu->tagprefix, 
+		  menu->tagprefix ? cur : idx[menu->current]->content);
         menu->redraw = REDRAW_FULL;
         break;
 #endif
@@ -976,16 +858,20 @@ void mutt_view_attachments (HEADER *hdr)
 
 
       case OP_PRINT:
-	mutt_print_attachment_list (fp, menu->tagprefix, menu->tagprefix ? cur : idx[menu->current]->content);
+	mutt_print_attachment_list (fp, menu->tagprefix, 
+		  menu->tagprefix ? cur : idx[menu->current]->content);
 	break;
 
       case OP_PIPE:
-	mutt_pipe_attachment_list (fp, menu->tagprefix, menu->tagprefix ? cur : idx[menu->current]->content, 0);
+	mutt_pipe_attachment_list (fp, menu->tagprefix, 
+		  menu->tagprefix ? cur : idx[menu->current]->content, 0);
 	break;
 
       case OP_SAVE:
-	mutt_save_attachment_list (fp, menu->tagprefix, menu->tagprefix ?  cur : idx[menu->current]->content, hdr);
-	if (option (OPTRESOLVE) && menu->current < menu->max - 1)
+	mutt_save_attachment_list (fp, menu->tagprefix, 
+		  menu->tagprefix ?  cur : idx[menu->current]->content, hdr);
+
+        if (option (OPTRESOLVE) && menu->current < menu->max - 1)
 	  menu->current++;
       
         menu->redraw = REDRAW_MOTION_RESYNCH | REDRAW_FULL;
@@ -994,6 +880,8 @@ void mutt_view_attachments (HEADER *hdr)
       case OP_DELETE:
 	CHECK_READONLY;
 
+
+
 #ifdef _PGPPATH
         if (hdr->pgp)
         {
@@ -1043,66 +931,65 @@ void mutt_view_attachments (HEADER *hdr)
         break;
 
       case OP_UNDELETE:
-	CHECK_READONLY;
-	if (!menu->tagprefix)
-	{
-	  idx[menu->current]->content->deleted = 0;
-	  if (option (OPTRESOLVE) && menu->current < menu->max - 1)
-	  {
-	    menu->current++;
-	    menu->redraw = REDRAW_MOTION_RESYNCH;
-	  }
-	  else
-	    menu->redraw = REDRAW_CURRENT;
-	}
-	else
-	{
-	  int x;
+       CHECK_READONLY;
+       if (!menu->tagprefix)
+       {
+	 idx[menu->current]->content->deleted = 0;
+	 if (option (OPTRESOLVE) && menu->current < menu->max - 1)
+	 {
+	   menu->current++;
+	   menu->redraw = REDRAW_MOTION_RESYNCH;
+	 }
+	 else
+	   menu->redraw = REDRAW_CURRENT;
+       }
+       else
+       {
+	 int x;
 
- 	  for (x = 0; x < menu->max; x++)
-	  {
-	    if (idx[x]->content->tagged)
-	    {
-	      idx[x]->content->deleted = 0;
-	      menu->redraw = REDRAW_INDEX;
-	    }
-	  }
-        }
-        break;
+	 for (x = 0; x < menu->max; x++)
+	 {
+	   if (idx[x]->content->tagged)
+	   {
+	     idx[x]->content->deleted = 0;
+	     menu->redraw = REDRAW_INDEX;
+	   }
+	 }
+       }
+       break;
 
+      case OP_RESEND:
+        CHECK_ATTACH;
+        mutt_attach_resend (fp, hdr, idx, idxlen,
+			     menu->tagprefix ? NULL : idx[menu->current]->content);
+        menu->redraw = REDRAW_FULL;
+      	break;
+      
       case OP_BOUNCE_MESSAGE:
         CHECK_ATTACH;
-	query_bounce_attachment (menu->tagprefix, menu->tagprefix ? cur : idx[menu->current]->content, hdr);
-	break;
+        mutt_attach_bounce (fp, hdr, idx, idxlen,
+			     menu->tagprefix ? NULL : idx[menu->current]->content);
+        menu->redraw = REDRAW_FULL;
+      	break;
 
+      case OP_FORWARD_MESSAGE:
+        CHECK_ATTACH;
+        mutt_attach_forward (fp, hdr, idx, idxlen,
+			     menu->tagprefix ? NULL : idx[menu->current]->content);
+        menu->redraw = REDRAW_FULL;
+        break;
+      
       case OP_REPLY:
       case OP_GROUP_REPLY:
       case OP_LIST_REPLY:
-      case OP_FORWARD_MESSAGE:
 
         CHECK_ATTACH;
-
-#ifdef _PGPPATH
-	if ((hdr->pgp & PGPENCRYPT) && hdr->content->type == TYPEMULTIPART)
-	{
-	  mutt_error _(
-	    "This operation is not currently supported for PGP messages.");
-	  break;
-	}
-#endif
-
-
-
-	if (op == OP_FORWARD_MESSAGE)
-	  flags = SENDFORWARD;
-	else
-	  flags = SENDREPLY | 
-		  (op == OP_GROUP_REPLY ? SENDGROUPREPLY : 0) |
-		  (op == OP_LIST_REPLY ? SENDLISTREPLY : 0);
-	reply_attachment_list (flags,
-			       menu->tagprefix,
-			       hdr,
-			       menu->tagprefix ? cur : idx[menu->current]->content);
+      
+        flags = SENDREPLY | 
+	  (op == OP_GROUP_REPLY ? SENDGROUPREPLY : 0) |
+	  (op == OP_LIST_REPLY ? SENDLISTREPLY : 0);
+        mutt_attach_reply (fp, hdr, idx, idxlen, 
+			   menu->tagprefix ? NULL : idx[menu->current]->content, flags);
 	menu->redraw = REDRAW_FULL;
 	break;
 
